@@ -400,6 +400,7 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
     int i, inum;
     int exp      = 0;
     int exp_sign = 1;
+    int negate = 0;
     char *lnp = get_localized_numeric_point();
     assert(lnp);
 
@@ -408,6 +409,18 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
     mpcim(&basevals[(int) base], MPbase);
 
     optr = str;
+
+    /* Remove any initial spaces or tabs. */
+    while (*optr == ' ' || *optr == '\t') {
+        optr++;
+    }
+
+    /* Check if this is a negative number. */
+    if (*optr == '-') {
+        negate = 1;
+        optr++;
+    }
+
     while ((inum = char_val(*optr)) >= 0) {
         mpmul(MPval, MPbase, MPval);
         mpaddi(MPval, &inum, MPval);
@@ -444,6 +457,10 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
         mppwr(MPbase, &exp, MP1);
         mpmul(MPval, MP1, MPval);
     }
+
+    if (negate == 1) {
+        mpneg(MPval, MPval);
+    }
 }
 
 
@@ -603,6 +620,22 @@ refresh_display()
 
                 localize_number(localized, ans);
                 str_replace(&str, "Ans", localized);
+
+                { /* Replace registers with values. */
+                    char reg[3];
+                    int n = '0';
+                    int i;
+
+                    for (i = 0; i < 10; i++) {
+                        char *reg_val;
+                        int MP_reg[MP_SIZE];
+
+                        snprintf(reg, 3, "R%c", n+i);
+                        do_rcl_reg(i, MP_reg);
+                        reg_val = make_number(MP_reg, v->base, FALSE);
+                        str_replace(&str, reg, reg_val);
+                    }
+                }
                 write_display(str);
                 free(str);
             } else {
